@@ -327,7 +327,7 @@ make_directories (name)
    existed.  */
 int
 mkdir_if_needed (name)
-    char *name;
+    const char *name;
 {
     if (mkdir (name, 0777) < 0)
     {
@@ -348,7 +348,7 @@ mkdir_if_needed (name)
  */
 void
 xchmod (fname, writable)
-    char *fname;
+    const char *fname;
     int writable;
 {
     struct stat sb;
@@ -610,6 +610,7 @@ xcmp (file1, file2)
 
     /* If FILE1 and FILE2 are symlinks, they are equal if they point to
        the same thing. */
+#ifdef S_ISLNK
     if (S_ISLNK (sb1.st_mode) && S_ISLNK (sb2.st_mode))
     {
 	int result;
@@ -620,6 +621,7 @@ xcmp (file1, file2)
 	free (buf2);
 	return result;
     }
+#endif
 
     /* If FILE1 and FILE2 are devices, they are equal if their device
        numbers match. */
@@ -767,8 +769,8 @@ FILE *cvs_temp_file (filename)
     if (fd == -1) fp = NULL;
     else if ((fp = CVS_FDOPEN (fd, "w+")) == NULL)
     {
-	/* attempt to close and unlink the file since mkstemp returned sucessfully and
-	 * we believe it's been created and opened
+	/* Attempt to close and unlink the file since mkstemp returned
+	 * sucessfully and we believe it's been created and opened.
 	 */
  	int save_errno = errno;
 	if (close (fd))
@@ -846,31 +848,33 @@ FILE *cvs_temp_file (filename)
     return fp;
 }
 
-/* Return non-zero iff FILENAME is absolute.
-   Trivial under Unix, but more complicated under other systems.  */
-int
-isabsolute (filename)
-    const char *filename;
-{
-    return filename[0] == '/';
-}
 
-/*
- * Return a string (dynamically allocated) with the name of the file to which
- * LINK is symlinked.
+
+#ifdef HAVE_READLINK
+/* char *
+ * xreadlink ( const char *link )
+ *
+ * Like the X/OPEN and 4.4BSD readlink() function, but allocates and returns
+ * its own buf.
+ *
+ * INPUTS
+ *  link	The original path.
+ *
+ * RETURNS
+ *  The resolution of the final symbolic link in the path.
+ *
+ * ERRORS
+ *  This function exits with a fatal error if it fails to read the link for
+ *  any reason.
  */
 char *
 xreadlink (link)
     const char *link;
 {
     char *file = NULL;
-    char *tfile;
     int buflen = 128;
     int link_name_len;
 
-    if (!islink (link))
-	return NULL;
-
     /* Get the name of the file to which `from' is linked.
        FIXME: what portability issues arise here?  Are readlink &
        ENAMETOOLONG defined on all systems? -twp */
@@ -887,19 +891,59 @@ xreadlink (link)
 
     file[link_name_len] = '\0';
 
-    tfile = xstrdup (file);
-    free (file);
+    return file;
+}
+#endif /* HAVE_READLINK */
 
-    return tfile;
+
+
+/* char *
+ * xresolvepath ( const char *path )
+ *
+ * Like xreadlink(), but resolve all links in a path.
+ *
+ * INPUTS
+ *  path	The original path.
+ *
+ * RETURNS
+ *  The path with any symbolic links expanded.
+ *
+ * ERRORS
+ *  This function exits with a fatal error if it fails to read the link for
+ *  any reason.
+ */
+char *
+xresolvepath ( path )
+    const char *path;
+{
+    char *hardpath;
+    char *owd;
+
+    assert ( isdir ( path ) );
+
+    /* FIXME - If HAVE_READLINK is defined, we should probably walk the path
+     * bit by bit calling xreadlink().
+     */
+
+    owd = xgetwd();
+    if ( CVS_CHDIR ( path ) < 0)
+	error ( 1, errno, "cannot chdir to %s", path );
+    if ( ( hardpath = xgetwd() ) == NULL )
+	error (1, errno, "cannot getwd in %s", path);
+    if ( CVS_CHDIR ( owd ) < 0)
+	error ( 1, errno, "cannot chdir to %s", owd );
+    free (owd);
+    return hardpath;
 }
 
 
+
 /* Return a pointer into PATH's last component.  */
-char *
+const char *
 last_component (path)
-    char *path;
+    const char *path;
 {
-    char *last = strrchr (path, '/');
+    const char *last = strrchr (path, '/');
     
     if (last && (last != path))
         return last + 1;
@@ -1001,11 +1045,11 @@ expand_wild (argc, argv, pargc, pargv)
    not).  */
 int
 cvs_casecmp (str1, str2)
-    char *str1;
-    char *str2;
+    const char *str1;
+    const char *str2;
 {
-    char *p;
-    char *q;
+    const char *p;
+    const char *q;
     int pqdiff;
 
     p = str1;
@@ -1020,185 +1064,3 @@ cvs_casecmp (str1, str2)
     return pqdiff;
 }
 #endif /* SERVER_SUPPORT */
-
-
-
-/* static char *
- * locate_file_in_dir ( char *dir, char *file )
- *
- * Search a directory for a filename, case insensitively when appropriate.
- *
- * INPUTS
- *
- *  dir		Path to directory to be searched.
- *  file	File name to be located, perhaps case insensitively.
- *
- * RETURNS
- *
- *  A newly malloc'd array containing the full path to the located file
- *  or NULL of the file was not found in dir or if the file found was
- *  not readable.
- *
- * ERRORS
- *
- *  When this function returns NULL, errno will be set appropriately.
- */
-static char *
-locate_file_in_dir ( dir, file )
-    char *dir;
-    char *file;
-{
-    char *retval;
-
-#if defined (SERVER_SUPPORT) && !defined (FILENAMES_CASE_INSENSITIVE)
-    if ( ign_case )
-    {
-	DIR *dirp;
-	struct dirent *dp;
-	char *found_name = NULL;
-
-	if ( ( dirp = CVS_OPENDIR ( dir ) ) == NULL )
-	{
-	    if ( existence_error ( errno ) )
-	    {
-		/* This can happen if we are looking in the Attic and the Attic
-		   directory does not exist.  Pass errno through to the caller;
-		   they know what to do with it.  */
-		return NULL;
-	    }
-	    else
-	    {
-		/* Give a fatal error; that way the error message can be
-		 * more specific than if we returned the error to the caller.
-		 */
-		error ( 1, errno, "cannot read directory %s", dir );
-	    }
-	}
-	errno = 0;
-	while ( ( dp = CVS_READDIR ( dirp ) ) != NULL )
-	{
-	    if ( cvs_casecmp ( dp->d_name, file ) == 0 )
-	    {
-		if ( found_name != NULL )
-		    error ( 1, 0, "%s is ambiguous; could mean %s or %s",
-			    file, dp->d_name, found_name );
-		found_name = xstrdup ( dp->d_name );
-	    }
-	}
-	if ( errno != 0 )
-	    error ( 1, errno, "cannot read directory %s", dir );
-
-	CVS_CLOSEDIR ( dirp );
-
-	if ( found_name == NULL )
-	{
-	    errno = ENOENT;
-	    return NULL;
-	}
-
-	/* Copy the found name back into DIR.  We are assuming that
-	   found_name is the same length as fname, which is true as
-	   long as the above code is just ignoring case and not other
-	   aspects of filename syntax.  */
-	retval = xmalloc ( strlen ( dir )
-			   + strlen ( found_name )
-			   + 2 );
-	sprintf ( retval, "%s/%s", dir, found_name );
-    }
-    else
-#endif /* defined (SERVER_SUPPORT) && !defined (FILENAMES_CASE_INSENSITIVE) */
-    {
-	retval = xmalloc ( strlen ( dir )
-			   + strlen ( file )
-			   + 2 );
-	(void) sprintf ( retval, "%s/%s", dir, file );
-	if ( !isfile ( retval ) )
-	{
-	    free ( retval );
-	    return NULL;
-	}
-    }
-
-    return retval;
-}
-
-
-
-/*
- * char *
- * locate_rcs ( const char* file, const char *repository , int *inattic )
- *
- * Find an RCS file in the repository.  Most parts of CVS will want to
- * rely instead on RCS_parse which calls this function and is
- * called by recurse.c which then puts the result in useful places
- * like the rcs field of struct file_info.
- *
- * INPUTS
- *
- *  repository		the repository (including the directory)
- *  file		the filename within that directory (without RCSEXT).
- *  inattic		NULL or a pointer to the output boolean
- *
- * OUTPUTS
- *
- *  inattic		If this input was non-null, the destination will be
- *  			set to true if the file was found in the attic or
- *  			false if not.  If no RCS file is found, this value
- *  			is undefined.
- *
- * RETURNS
- *
- *  a newly-malloc'd array containing the absolute pathname of the RCS
- *  file that was found or NULL on error.
- *
- * ERRORS
- *
- *  errno will be set by the system calls in the case of failure.
- */
-char *
-locate_rcs ( repository, file, inattic )
-    const char *repository;
-    const char *file;
-    int *inattic;
-{
-    char *rcsfile;
-    char *dir;
-    char *retval;
-
-    /* Allocate space and add the RCS extension */
-    rcsfile = xmalloc ( strlen ( file )
-		    + sizeof ( RCSEXT ) );
-    (void) sprintf ( rcsfile, "%s%s", file, RCSEXT );
-
-    /* Search in the top dir given */
-    if (( retval = locate_file_in_dir ( repository, rcsfile )) != NULL )
-    {
-	if ( inattic )
-	    *inattic = 0;
-	goto out;
-    }
-
-    /* Search in the Attic */
-    dir = xmalloc ( strlen ( repository )
-		    + sizeof ( CVSATTIC )
-		    + 2 );
-    (void) sprintf ( dir,
-		     "%s/%s",
-		     repository,
-		     CVSATTIC );
-
-    if ( ( retval = locate_file_in_dir ( dir, rcsfile ) ) != NULL
-	 && inattic != NULL )
-	*inattic = 1;
-
-    free ( dir );
-
-out:
-    free ( rcsfile );
-    return retval;
-}
-
-
-
-/* vim:tabstop=8:shiftwidth=4
- */
